/* ANSICHEK.C -- test whether ANSI.SYS is installed. */
/* Will compile with Borland or Microsoft compilers. */
/* This code is hereby released to the public domain */
/* and may be changed at the user's discretion.      */

// Last Mod by:  Allen Jackson, Fri  10-04-1991  16:18:36
// Changed to ANSICHEK v1.10
// Archived as ANSC110

// Returns ERRORLEVEL 1 if ANSI.SYS is not active.
//
// Test in a batch file like:
//
// @ECHO OFF
// ANSICHEK
// IF ERRORLEVEL 1 GOTO skip
// ECHO [41;1;37m[2J[12;30fWow!  Look at the color.[0m
// GOTO exit
// :skip
// ECHO Sorry, couldn't change the color.
// :exit

#include <stdio.h>
#include <dos.h>

#define ACTIVE          (0)
#define NOTACT          (1)
#define INPUT           (0xff)
#define ESC             ('\x1b')
#define DSR             ("\x1b[6n")
#define TINY            unsigned char

// Function prototypes
void exit(int);
void EchoString(char *);
TINY DirectConsoleIO(TINY);
void ForceDupConsole(void);
void RestoreConsole(void);

static char *active = ("\x1b[4;1mANSI\x1b[0m active!\n");
static char *notact = ("\b\b\b\b\bANSI not active.\n");

TINY rc;

void main(void)
{
     ForceDupConsole();                      // AJ 10-04-91

     EchoString(DSR);                        // ANSI Device Status Request
     rc = DirectConsoleIO((TINY)INPUT);      // retrieve the next character

     while (DirectConsoleIO((TINY)INPUT));   // eat RCP resulting string
     if (rc == ESC)                          // if ESC was received
     {
          EchoString(active);                // ANSI active
          rc = ACTIVE;                       // set exit code
     }
     else
     {
          EchoString(notact);                // not active -- erase DSR
          rc = NOTACT;                       // set exit code
     }
     RestoreConsole();                       // AJ 10-04-91
     exit(rc);
}

// EchoString: Print a string to the directly to the console via
// DirectConsoleIO

void EchoString(char *str)
{
     int i;
     for (i = 0; str[i]; DirectConsoleIO(str[i++]));
}

// DirectConsoleIO: Use DOS INT21h function 06h to sent char to console

TINY DirectConsoleIO(TINY charout)
{
     _asm {
          mov  ah, 06h              // Function 06h
          mov  dl, charout          // Put charout into DL
          int  21h                  // DOS Interrupt 21h
          mov  charout, al          // Put return char into charout
     }
     return charout;
}

// ForceDupConsole: Place console on a duplicate handle to avoid redirection
// AJ 10-04-91

void ForceDupConsole(void)
{
     static int oldhandle;

     _asm {
          mov  ah, 45h              // Function 45h (Dup Handle)
          mov  bx, 01h              // Stdout handle
          int  21h
          mov  oldhandle, ax

          mov  ah, 46h              // Function 46h (Force Dup Handle)
          mov  bx, 02h              // Stderr handle
          mov  cx, 01h              // Stdout handle
          int  21h
     }
     return;
}

// RestoreConsole: Restore console to previous handle
// AJ 10-04-91

void RestoreConsole(void)
{
     static int oldhandle;

     _asm {
          mov  ah, 46h              // Function 46h (Force Dup Handle)
          mov  bx, oldhandle
          mov  cx, 01h              // Stdout handle
          int  21h

          mov  ah, 3eh              // Function 3eh (Close)
          mov  bx, oldhandle
          int  21h
     }
     return;
}

